[CmdletBinding()]
Param(
    [switch] $TestSigning,
    [string] $MetaDataRoot,
    [string] $UnsignedMetaDataPath,
    [switch] $DoNotElevate,
    [switch] $RecurseToMetadata
)

Function GetArgumentList() {
    $ArgumentList = ''

    if($TestSigning) {
        $ArgumentList += ' -TestSigning'
    }

    if( ($MetaDataRoot -ne $Null) -and ($MetaDataRoot -ne "")) {
        $ArgumentList += " -MetaDataRoot $MetaDataRoot"
    }

    if( ($UnsignedMetaDataPath -ne $Null) -and ($UnsignedMetaDataPath -ne "")) {
        $ArgumentList += " -UnsignedMetaDataPath $UnsignedMetaDataPath"
    }

    if($RecurseToMetadata) {
        $ArgumentList += ' -RecurseToMetadata'
    }

    return $ArgumentList
}

####
# This script is included in test release and
#  used to install the metadata and application
# 
# Usage: powershell InstallMetadata.ps1 [-TestSigning] [-MetaDataRoot] [-UnsignedMetaDataPath] [-RecurseToMetadata]
####

$currentDirectory = Get-Location

$metadataCache="c:\ProgramData\Microsoft\Windows\DeviceMetadataCache\"
$metadataStore="c:\ProgramData\Microsoft\Windows\DeviceMetadataStore\en-US\"

# Define the metadata root folder
if( ($MetaDataRoot -eq $Null) -or ($MetaDataRoot -eq ""))
{
    $metadataSource = ".\MetaData\"
}
else {
    $metadataSource = $MetaDataRoot
}

# Figure out the path from root folder to metadata packages which are not signed.
# FIXME: This is bit clumsy, but good enough for now
if( ($UnsignedMetaDataPath -eq $Null) -or ($UnsignedMetaDataPath -eq ""))
{
    $unsignedMetadataPath = "TestSigning"
}
else {
    $unsignedMetadataPath = $UnsignedMetaDataPath
}

function exitIfNoMetaDataIsAvailable(){
    IF (!(Test-Path $metadataSource)){
        Write-Host "No MetaData folder found from the directory [$metadataSource]"
        Write-Host "Note: this might be because you didn't start this as an admin."
        $tmp = Read-host "Hit enter to continue"
        exit 1
    }
}

function isPackageInstalled($filename){
	$counter = Get-ChildItem -Path $metadataStore -recurse -include $filename |Measure-Object
	Return $counter.Count -gt 0
}

function installMetadata(){
    exitIfNoMetaDataIsAvailable

    $getFilesExpr = "Get-ChildItem " + $metadataSource + "\* -Include *.devicemetadata-ms"

    IF($RecurseToMetadata) {
        $getFilesExpr = $getFilesExpr + " -Recurse"
    }

    Write-Verbose "running command $getFilesExpr"

    Invoke-Expression $getFilesExpr | Foreach {
        IF(!(isPackageInstalled $_.Name)) {
            Copy-Item $_ $metadataStore"\"
        }
    }
}

function prepareForUnsignedMetadata(){
    Write-Warning "Continuing on some tablets which have Bitlocker encryption on, can cause the tablet to turn into a brick."
    Write-Warning "This will clear all MobileBroadband related entries from registry (from HKLM:\SOFTWARE\Microsoft\WwanSvc\MobileBroadbandAccounts\)"
    Write-Warning "This will clear the whole DeviceMetadataCache folder and old Metadata packages from DeviceMetadataStore (only our own)"
    Write-Warning "This will set the Vodafone Preview key to allow downloading OTA preview Metadata"
    $answer = Read-host "Continue Y/N"
    IF ($answer -eq "Y") {
       bcdedit /set testsigning on
       Write-Host "set testsigning on"
       Set-Variable -Name metadataSource -Value "$metadataSource\$unsignedMetadataPath" -Scope 1              
       addTestKey
       Write-Host "enabled preview OTA metadata"
    }
	ELSE {exit 1}
}

function restartIfNeeded(){	
	Write-Host "A restart is required for the changes to take affect. Note: if you had trouble detecting SIM card previously, restart is needed."
	$answer = Read-Host "Restart now? Y/N"
	IF ($answer -eq "Y"){
		Restart-Computer;
	}
}


function clearRegistryAndMetadata(){
    # clear metadatacache
    Get-ChildItem -Recurse $metadataCache"\"* | Remove-Item -Force -Recurse

    # remove reg keys
    Get-ChildItem HKLM:\SOFTWARE\Microsoft\WwanSvc\MobileBroadbandAccounts\Accounts -ErrorAction SilentlyContinue | Remove-Item -Force
    Get-ChildItem HKLM:\SOFTWARE\Microsoft\WwanSvc\MobileBroadbandAccounts\NetworkInterfaceBindings -ErrorAction SilentlyContinue | Remove-Item -Force

    # clearing out all metadata packages created by us
    Write-Verbose "Removing metadata packages based on *.lst files"
    dir .\*metadatapackages.lst | Foreach {
        Write-Verbose "Handling file $_"
        Get-Content $_ | Foreach-Object {
            IF (Test-Path $metadataStore"\"$_) {
                Remove-Item $metadataStore"\"$_
            }
        }
	}
}

function addTestKey(){
    #set the preview key for metadata
    New-ItemProperty -Path "HKLM:\SOFTWARE\Microsoft\Windows\CurrentVersion\Device Metadata\" -Force -Name "DeviceMetadataPreviewKey" -Value "vodafonetest" -PropertyType "String"
}

function removeTestKey(){
    #remove the preview key for metadata
    Remove-ItemProperty -Path "HKLM:\SOFTWARE\Microsoft\Windows\CurrentVersion\Device Metadata\" -Force -Name "DeviceMetadataPreviewKey" -ErrorAction SilentlyContinue
}

####
# Elevating to administrator privilges
####

#
# Relaunch the process with elevated privileges. Set -DoNotElevate argument to prevent
# infinite relaunch loop
#
function LaunchElevated() {
    Write-Host "This script requires admin privileges. Starting new process with elevated privileges"

    # Set up command line arguments to the elevated process
    $RelaunchArgs = '-executionpolicy bypass -file "' + $ScriptPath + '"'

    $RelaunchArgs += ' -DoNotElevate'

    # Add the original arguments
    $RelaunchArgs += GetArgumentList

    # Launch the process and wait for it to finish
    try
    {
        $AdminProcess = Start-Process "$PsHome\PowerShell.exe" -Verb RunAs -ArgumentList $RelaunchArgs -PassThru
    }
    catch
    {
        $Error[0] # Dump details about the last error
        Write-Host "Failed to launch admin process"
    }

    while (!($AdminProcess.HasExited))
    {
        Start-Sleep -Seconds 2
    }

    # The execution of the script returns here after the process has exited
}

function checkDeveloperLicense() {
    $private:isValid = isDeveloperLicenseValid
    IF (!$isValid) {
        Write-Warning "Developer license not valid, please renew it."
        renewDeveloperLicense
    }
    ELSE {
        Write-Verbose "Developer license valid, continuing normally."
    }
}

function renewDeveloperLicense() {
    Show-WindowsDeveloperLicenseRegistration
}

function isDeveloperLicenseValid() {
    $val = Get-WindowsDeveloperLicense

    return $val.IsValid
}

function DoElevatedOperations(){
    checkDeveloperLicense

    # Empty metadata store and cache
    clearRegistryAndMetadata
    Write-Host "Cleared old metadata and registry"
	
    # Check for TestSigning
    IF ($TestSigning){
        prepareForUnsignedMetadata
    }
    ELSE {
        removeTestKey
    }

    # Install metadata
    installMetadata

    # Done
    Write-Host "--- Installation done ---"
    restartIfNeeded
}

####
# Get Invocation information
####

$ScriptPath = $null
try
{
    $ScriptPath = (Get-Variable MyInvocation).Value.MyCommand.Path
}
catch {}

if (!$ScriptPath)
{
    Throw "Can not find script path"
}

####
# Main access point
####

Write-Host "                _        __                  "
Write-Host "__   _____   __| | __ _ / _| ___  _ __   ___ "
Write-Host "\ \ / / _ \ / _' |/ _' | |_ / _ \| '_ \ / _ \"
Write-Host " \ V / (_) | (_| | (_| |  _| (_) | | | |  __/"
Write-Host "  \_/ \___/ \__,_|\__,_|_|  \___/|_| |_|\___|"
Write-Host "                                             "

if ($DoNotElevate) {
    DoElevatedOperations
}
else 
{
    LaunchElevated
}

Write-Host "Done."
